@@ -35,6 +35,7 @@
 #include "config.h"
 #endif
 
+#include <unistd.h>
 #include <stdlib.h>
 
 #include <glib.h>
@@ -55,23 +56,49 @@ struct _ClutterIMTextPrivate
 
 G_DEFINE_TYPE (ClutterIMText, clutter_imtext, CLUTTER_TYPE_TEXT)
 
-static gboolean
-_get_env_enable_autoshow(void)
-{
-  static gint env_enable_autoshow = -1;
 
-  if (env_enable_autoshow == -1) {
+static gboolean
+get_autoshow_sysconf (void)
+{
+  gchar *path = NULL;
+  gboolean ret = FALSE;
+
+  path = g_build_filename (CLUTTER_IMCONTEXT_SYSCONFDIR,
+                           "clutter-imcontext", "enable_autoshow", NULL);
+  if (g_access(path, F_OK) == 0)
+    ret = TRUE;
+
+  g_free(path);
+
+  return ret;
+}
+
+
+static gboolean
+_get_enable_autoshow(void)
+{
+  static gint enable_autoshow = -1;
+
+  if (enable_autoshow == -1) {
 
     const gchar *envvar = NULL;
 
     envvar = g_getenv ("CLUTTER_IMCONTEXT_AUTOSHOW");
+    if (!envvar) {
+      if (get_autoshow_sysconf()) {
+        enable_autoshow = 1;
+        goto done;
+      }
+    }
+
     if (envvar && !g_ascii_strcasecmp (envvar, "TRUE"))
-      env_enable_autoshow = 1;
+      enable_autoshow = 1;
     else
-      env_enable_autoshow = 0;
+      enable_autoshow = 0;
   }
 
-  return (env_enable_autoshow == 1);
+done:
+  return (enable_autoshow == 1);
 }
 
 /**
@@ -88,7 +115,7 @@ _get_env_enable_autoshow(void)
  */
 static gboolean
 clutter_text_position_to_pixel (ClutterText *text,
-                                 gint         position,
+                                 gint position,
                                  gint *x, gint *y,
                                  gint *line_height)
 {
@@ -124,7 +151,7 @@ static void update_im_cursor_location (ClutterIMText *self)
 
 static void
 clutter_imtext_commit_cb (ClutterIMContext *context, const gchar  *str,
-				ClutterIMText *imtext)
+                          ClutterIMText *imtext)
 {
   ClutterIMTextPrivate *priv = imtext->priv;
   ClutterText *clutter_text = CLUTTER_TEXT (imtext);
@@ -182,12 +209,12 @@ clutter_imtext_paint (ClutterActor *actor)
 }
 
 static gboolean
-clutter_imtext_key_press (ClutterActor    *actor,
-			  ClutterKeyEvent *event)
+clutter_imtext_key_press_cb (ClutterActor *actor,
+                             ClutterKeyEvent *event,
+                             ClutterIMText *imtext)
 {
-  ClutterIMText *self = CLUTTER_IMTEXT (actor);
-  ClutterIMTextPrivate *priv = self->priv;
-  ClutterText *clutter_text = CLUTTER_TEXT (actor);
+  ClutterIMTextPrivate *priv = imtext->priv;
+  ClutterText *clutter_text = CLUTTER_TEXT (imtext);
 
   if (!clutter_text_get_editable(clutter_text))
     return FALSE;
@@ -195,19 +222,16 @@ clutter_imtext_key_press (ClutterActor    *actor,
   if (clutter_im_context_filter_keypress (priv->im_context, event))
     return TRUE;
 
-  if (CLUTTER_ACTOR_CLASS (clutter_imtext_parent_class)->key_press_event)
-    return CLUTTER_ACTOR_CLASS (clutter_imtext_parent_class)->key_press_event (actor, event);
-
   return FALSE;
 }
 
 static gboolean
-clutter_imtext_key_release (ClutterActor    *actor,
-			    ClutterKeyEvent *event)
+clutter_imtext_key_release_cb (ClutterActor    *actor,
+                               ClutterKeyEvent *event,
+                               ClutterIMText *imtext)
 {
-  ClutterIMText *self = CLUTTER_IMTEXT (actor);
-  ClutterIMTextPrivate *priv = self->priv;
-  ClutterText *clutter_text = CLUTTER_TEXT (actor);
+  ClutterIMTextPrivate *priv = imtext->priv;
+  ClutterText *clutter_text = CLUTTER_TEXT (imtext);
 
   if (!clutter_text_get_editable(clutter_text))
     return FALSE;
@@ -215,9 +239,6 @@ clutter_imtext_key_release (ClutterActor    *actor,
   if (clutter_im_context_filter_keypress (priv->im_context, event))
     return TRUE;
 
-  if (CLUTTER_ACTOR_CLASS (clutter_imtext_parent_class)->key_release_event)
-    return CLUTTER_ACTOR_CLASS (clutter_imtext_parent_class)->key_release_event (actor, event);
-
   return FALSE;
 }
 
@@ -230,8 +251,6 @@ clutter_imtext_class_init (ClutterIMTextClass *klass)
   g_type_class_add_private (klass, sizeof (ClutterIMTextPrivate));
 
   actor_class->paint = clutter_imtext_paint;
-  actor_class->key_press_event = clutter_imtext_key_press;
-  actor_class->key_release_event = clutter_imtext_key_release;
 }
 
 static void
@@ -247,9 +266,15 @@ clutter_imtext_init (ClutterIMText *self)
   g_signal_connect (self, "key-focus-out",
                     G_CALLBACK (clutter_imtext_focus_out_cb), self);
 
+  g_signal_connect (self, "key-press-event",
+                    G_CALLBACK (clutter_imtext_key_press_cb), self);
+
+  g_signal_connect (self, "key-release-event",
+                    G_CALLBACK (clutter_imtext_key_release_cb), self);
+
   priv->im_context = clutter_im_multicontext_new ();
-  priv->im_context->actor = CLUTTER_ACTOR(self);
-  priv->auto_show_im = _get_env_enable_autoshow();
+  priv->im_context->actor = CLUTTER_ACTOR (self);
+  priv->auto_show_im = _get_enable_autoshow ();
 
   g_signal_connect (priv->im_context, "commit",
 		    G_CALLBACK (clutter_imtext_commit_cb), self);
@@ -267,8 +292,8 @@ ClutterActor *
 clutter_imtext_new (const gchar *text)
 {
   return g_object_new (CLUTTER_TYPE_IMTEXT,
-                          "text", text,
-                          NULL);
+                       "text", text,
+                       NULL);
 }
 
 /**
